import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.ArrayList;

import jec.ExchangeConnectorFactory;
import jec.ExchangeConnectorInterface;
import jec.ExchangeConstants;
import jec.ExchangeGeneralException;
import jec.dto.AttachmentDTO;
import jec.dto.ExchangeEmailDTO;


public class MsgParserExamples {

	public static void main(String[] args) {
		try {
			parseEmailDTOFromMsgFileExample("./resources/test_emails/test7.msg");
			parseEmailDTOFromEmlFileExample("./resources/test_emails/test1.eml");
		} catch (ExchangeGeneralException e) {
			e.printStackTrace();
		}
	}


	public static void parseEmailDTOFromMsgFileExample(String filePath2parse) throws ExchangeGeneralException {
		System.out.println("============== parseEmailDTOFromMsgFile start ===========");
		ExchangeConnectorFactory factory = new ExchangeConnectorFactory();
		ExchangeConnectorInterface connector = null;
		connector = factory.createExchangeConnector();
		connector.setEmailHandlerType(ExchangeConstants.EMAIL_HANDLER_TYPE_FETCHER);
		ExchangeEmailDTO email = connector.parseEmailDTOFromEmailMsgFile(filePath2parse);
		System.out.println("subject: " + email.getSubject());
		System.out.println("Recieved date: " + email.getDateReceived());
		System.out.println("Recieved date Str: " + email.getDateReceivedStr());
		if (email.getFromAsAddressDTO() != null)
			System.out.println("from: " + email.getFromAsAddressDTO().getName() + " " + email.getFromAsAddressDTO().getEmailAddress() );



		if (email.getToEmailNames() != null && email.getToEmailNames().length > 0) {
			System.out.println("first To name: " + email.getToEmailNames()[0]);
		}
		if (email.getToEmailAddresses() != null && email.getToEmailAddresses().length > 0) {
			System.out.println("first To Address: " + email.getToEmailAddresses()[0]);
		}
		if (email.getCcEmailNames() != null && email.getCcEmailNames().length > 0) {
			System.out.println("first Cc name: " + email.getCcEmailNames()[0]);
		}
		if (email.getCcEmailAddresses() != null && email.getCcEmailAddresses().length > 0) {
			System.out.println("first Cc address: " + email.getCcEmailAddresses()[0]);
		}

		if (email.getBccEmailNames() != null && email.getBccEmailNames().length > 0) {
			System.out.println("first Bcc name: " + email.getBccEmailNames()[0]);
		}
		if (email.getBccEmailAddresses() != null && email.getBccEmailAddresses().length > 0) {
			System.out.println("first Bcc Addresses: " + email.getBccEmailAddresses()[0]);
		}

		System.out.println("=====================================================================================");
		System.out.println("body: " + email.getBody());
		System.out.println("=====================================================================================");

		ArrayList attachments = email.getAttachmentsAsAttachmentDTOArrayList();
		if ((attachments != null) && (attachments.size() > 0)) {
			System.out.println("number of attachments: " + email.getAttachmentsAsAttachmentDTOArrayList().size());
			AttachmentDTO attachment = (AttachmentDTO)attachments.get(0);
			System.out.println("getting the first one: " + attachment.getName() );
			if (attachment.getInputStreamContent() != null)
			  saveFile(attachment.getName(), attachment.getInputStreamContent());
			else if (attachment.getByteData() != null)
				saveFile(attachment.getName(), attachment.getByteData());
		}
		System.out.println("============== parseEmailDTOFromMsgFile end ===========");
	}

	public static void parseEmailDTOFromEmlFileExample(String filePath2parse) throws ExchangeGeneralException {
		System.out.println("============== parseEmailDTOFromEmlFile start ===========");
		ExchangeConnectorFactory factory = new ExchangeConnectorFactory();
		ExchangeConnectorInterface connector = null;
		connector = factory.createExchangeConnector();
		connector.setEmailHandlerType(ExchangeConstants.EMAIL_HANDLER_TYPE_FETCHER);
		ExchangeEmailDTO email = connector.parseEmailDTOFromEmailMsgFile(filePath2parse);
		System.out.println("subject: " + email.getSubject());
		System.out.println("Recieved date: " + email.getDateReceived());
		System.out.println("Recieved date Str: " + email.getDateReceivedStr());
		if (email.getFromAsAddressDTO() != null)
			System.out.println("from: " + email.getFromAsAddressDTO().getName() + " " + email.getFromAsAddressDTO().getEmailAddress() );

		if (email.getToArray() != null)
			System.out.println("first To: " + email.getToArray()[0].getFullAddressStr());
		if (email.getCcArray() != null)
			System.out.println("first Cc: " + email.getCcArray()[0].getFullAddressStr());
		if (email.getBccArray() != null)
			System.out.println("first Bcc: " + email.getBccArray()[0].getFullAddressStr());

		System.out.println("=====================================================================================");
		System.out.println("body: " + email.getBody());
		System.out.println("=====================================================================================");

		ArrayList attachments = email.getAttachmentsAsAttachmentDTOArrayList();
		if ((attachments != null) && (attachments.size() > 0)) {
			System.out.println("number of attachments: " + email.getAttachmentsAsAttachmentDTOArrayList().size());
			AttachmentDTO attachment = (AttachmentDTO)attachments.get(0);
			System.out.println("getting the first one: " + attachment.getName() );
			if (attachment.getInputStreamContent() != null)
			  saveFile(attachment.getName(), attachment.getInputStreamContent());
			else if (attachment.getByteData() != null)
				saveFile(attachment.getName(), attachment.getByteData());
		}
		System.out.println("============== parseEmailDTOFromEmlFile end ===========");
	}

	/**
     * The file will be saved to JEC's root
     * @param filename String
     * @param is InputStream
     */
    private static void saveFile(String filename, InputStream is) {

        File file = new File(filename);
        try {
            FileOutputStream fos = new FileOutputStream(filename);
            byte[] buf = new byte[256];
            int read = 0;
            while ((read = is.read(buf)) > 0) {
                fos.write(buf, 0, read);
            }
            System.out.println("saved to file Successfully");
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }


    }


    /**
     * The file will be saved to JEC's root
     * @param filename String
     * @param is InputStream
     */
    private static void saveFile(String filename, byte[] content) {

        File file = new File(filename);
        try {
            FileOutputStream fos = new FileOutputStream(filename);
            fos.write(content);
            System.out.println("saved to file Successfully");
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }


    }

}
